<?xml version="1.0" encoding="UTF-8"?>
<?xml-stylesheet type="text/xsl" href="sch2html.xsl"?>
<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xhtml="http://www.w3.org/1999/xhtml">

    <sch:title>Rules for an SDMX-HD Data Structure Definition</sch:title>
    <sch:p>This document contains rules about an SDMX-HD Data Structure Definition. It does not
        replicate existing constraints - which should be validated with the SDMX W3C schemas. It
        does provide additional constraints which conforming producers of an SDMX-HD DSD shall
        comply with.</sch:p>

    <sch:p>Each rule consists of a context and a list of assertions. The context is expressed as a
        W3C XPath 1.0 expression, which identifies one or more elements within a conforming DSD
        document. The assertions are natual language assertions which must be true in the given
        context. A test for each assertion is also provided as a W3C XPath 1.0 expression.</sch:p>
    <sch:p>The XPath expresions may refer to key elements from the XSLT 1.0 namespace. The list of
        such XSLT keys declared is given below.</sch:p>

    <sch:ns uri="http://www.SDMX.org/resources/SDMXML/schemas/v2_0/message" prefix="msg"/>
    <sch:ns uri="http://www.SDMX.org/resources/SDMXML/schemas/v2_0/structure" prefix="st"/>


    <xsl:key name="volatile_aliases"
        match="//st:HierarchicalCodelist[@id='HCL_CONFIGURATION_HIERARCHIES']/st:CodelistRef"
        use="st:Alias"/>

    <xsl:key name="dimensions" match="//st:Dimension" use="@codelist"/>

    <sch:pattern id="general-dsd">
        <sch:title>General rules for DSD structure message</sch:title>
        <sch:p>An SDMX DSD message is a valid SDMX Structure message which has additional
            constraints imposed upon it. </sch:p>

        <sch:rule context="/msg:Structure" role="Required elements of a DSD">
            <sch:assert test="msg:KeyFamilies/st:KeyFamily" flag="fatal">A DSD document shall have
                at least one KeyFamily element from the SDMX message namespace. </sch:assert>
            <sch:assert test="count(msg:CodeLists)=1" flag="fatal">A DSD document shall have one
                CodeLists element from the SDMX message namespace. </sch:assert>
            <sch:assert test="count(msg:Concepts)=1" flag="fatal">A DSD document shall have one
                Concepts element from the SDMX message namespace. </sch:assert>
            <sch:assert test="count(msg:HierarchicalCodelists)=1" flag="fatal">A DSD document shall
                have one HierarchicalCodelists element from the SDMX message namespace. </sch:assert>
        </sch:rule>
    </sch:pattern>

    <sch:pattern>
        <sch:title>General rules for HierarchicalCodeLists</sch:title>
        <sch:rule context="msg:HierarchicalCodelists" role="Required HierarchicalCodeLists">
            <sch:assert flag="fatal"
                test="count(st:HierarchicalCodelist[@id='HCL_CONFIGURATION_HIERARCHIES' 
          and @isFinal='false'])=1"
                >It is an error if there is not a HierarchicalCodelist with
                id=HCL_CONFIGURATION_HIERARCHIES and isFinal=false.</sch:assert>
        </sch:rule>
        <sch:rule role="CodeList aliases" context="//st:CodeRef">
            <sch:let name="parent" value="ancestor::st:HierarchicalCodelist"/>
            <sch:let name="alias" value="st:CodelistAliasRef"/>
            <sch:assert flag="fatal" test="count($parent/st:CodelistRef[st:Alias=$alias])=1">
                It is an error if a CodelistAliasRef does not refer to an 
                alias of a codelist defined in the parent HierarchicalCodeList. </sch:assert>
        </sch:rule>
    </sch:pattern>
 
    <sch:pattern>
        <sch:title>Rules for configuration hierarchies</sch:title>
        <sch:rule context="st:HierarchicalCodelist[@id='HCL_CONFIGURATION_HIERARCHIES']"
            role="Required hierarchies">
            <sch:assert flag="fatal" test="count(st:Hierarchy[@id='HY_INDICATOR_DISAGGREGATION'])=1"> It is
                an error if there is not a Hierarchy within the configuration hierarchical codelist 
                with id=HY_INDICATOR_DISAGGREGATION. </sch:assert>
            </sch:rule>
            <sch:rule context="st:Hierarchy" role="Permitted hierarchies">
            <sch:assert flag="fatal" test="@id='HY_TIME_PERIOD' or @id='HY_INDICATOR' 
                or @id='HY_PROGRAM' or @id='HY_GEOGRAPHIC_PLACE_NAME' or
                @id='HY_INDICATOR_DSTYPE' or @id='HY_INDICATOR_SET' or 
                @id='HY_ORGANIZATION_CONTACT' or @id='HY_INDICATOR_DISAGGREGATION'"> The configuration
                hierarchical codelist may contain Hierarchies with id=HY_TIME_PERIOD, HY_INDICATOR, HY_PROGRAM,
                HY_GEOGRAPHIC_PLACE_NAME, HY_INDICATOR_DSTYPE, HY_INDICATOR_SET or HY_ORGANIZATION_CONTACT.</sch:assert>
        </sch:rule>
    </sch:pattern>
    
    <sch:pattern>
        <sch:title>Rules for indicator disaggregation hierarchy</sch:title>
        <sch:rule context="st:Hierarchy[@id='HY_INDICATOR_DISAGGREGATION']/st:CodeRef"
            role="Indicator disaggregation">
            <sch:assert flag="fatal" test="key('volatile_aliases',st:CodelistAliasRef)/st:CodelistID='CL_PROGRAM'"> It is an error if the
                first child node of the Hierarchy with id='HY_INDICATOR_DISAGGREGATION' is not a
                Program node (CodelistAliasRef points to CL_PROGRAM). </sch:assert>
        </sch:rule>
        
        <sch:rule
            context="st:Hierarchy[@id='HY_INDICATOR_DISAGGREGATION']/st:CodeRef/st:CodeRef"
            role="The indicator set node">
            <sch:assert flag="fatal" test="key('volatile_aliases',st:CodelistAliasRef)/st:CodelistID='CL_ISET'"> It is an error if an
                immediate child node of a Program node within the Hierarchy with
                id='HY_INDICATOR_DISAGGREGATION' is not an Indicator Set node (CodelistAliasRef points to CL_ISET) </sch:assert>
        </sch:rule>
        
        <sch:rule
            context="st:Hierarchy[@id='HY_INDICATOR_DISAGGREGATION']/st:CodeRef/st:CodeRef/st:CodeRef"
            role="The indicator node">
            <sch:assert flag="fatal" test="key('volatile_aliases',st:CodelistAliasRef)/st:CodelistID='CL_INDICATOR'"> It is an error if an
                an immediate child nodes of an Indicator Set node within the Hierarchy with
                id='HY_INDICATOR_DISAGGREGATION' is not an Indicator node (CodelistAliasRef points to CL_INDICATOR) </sch:assert>
        </sch:rule>
        
        <sch:rule
            context="st:Hierarchy[@id='HY_INDICATOR_DISAGGREGATION']/st:CodeRef/st:CodeRef/st:CodeRef/st:CodeRef"
            role="The value type node">
            <sch:assert flag="fatal" test="key('volatile_aliases',st:CodelistAliasRef)/st:CodelistID='CL_VALUE_TYPE'"> It is an error if
                an immediate child node of an Indicator node within the Hierarchy with
                id='HY_INDICATOR_DISAGGREGATION' is not a Value type node (CodelistAliasRef points to CL_VALUE_TYPE) </sch:assert>
        </sch:rule>
        
        <sch:rule
            context="st:Hierarchy[@id='HY_INDICATOR_DISAGGREGATION']/st:CodeRef/st:CodeRef/st:CodeRef/st:CodeRef/st:CodeRef"
            role="The first disaggregation node">
            <sch:assert flag="fatal"
                test="key('volatile_aliases',st:CodelistAliasRef)/st:CodelistID='CL_DISAGG' or count(key('dimensions',key('volatile_aliases',st:CodelistAliasRef)/st:CodelistID))=1"
                > It is an error if an immediate child node of an value type node within the
                Hierarchy with id='HY_INDICATOR_DISAGGREGATION' is not either a disaggregation-list
                node (CodelistAliasRef points to CL_DISAGG) or a node whose CodelistAliasRef refers to an alias of a codelist
                which is used as a Dimension in the KeyFamily </sch:assert>
        </sch:rule>
        
        <!--        This rule matches the remaining CodeRefs not matched by the rules above-->
        <sch:rule
            context="st:Hierarchy[@id='HY_INDICATOR_DISAGGREGATION']/st:CodeRef/st:CodeRef/st:CodeRef/st:CodeRef/st:CodeRef/st:CodeRef"
            role="Disaggregation nodes">
            <sch:assert flag="fatal"
                test="count(key('dimensions',key('volatile_aliases',st:CodelistAliasRef)/st:CodelistID))=1"
                >It is an error if a CodelistAliasRef refers to an alias of a codelist which is not
                used as a Dimension in the KeyFamily </sch:assert>
        </sch:rule>
    </sch:pattern>

    <sch:pattern>
        <sch:title>Rules for time period hierarchy</sch:title>
        <sch:p>A DSD may specify that the time dimension of the keyfamily makes use of a code list.  This hierarchy
        provides producers with a mechanism to structure the time periods.</sch:p>
        <sch:rule context="st:HierarchicalCodelist[@id='HCL_CONFIGURATION_HIERARCHIES']/st:Hierarchy[@id='HY_TIME_PERIOD']//st:CodeRef" 
            role="Time period nodes">
            <sch:assert test="key('volatile_aliases',st:CodelistAliasRef)/st:CodelistID='CL_TIME_PERIOD'">HY_TIME_PERIOD is a tree of time period nodes.  It is an error if a
                child node in this hierarchy does not have a CodelistAliasRef element which is an alias of 'CL_TIME_PERIOD'.</sch:assert>
        </sch:rule>
    </sch:pattern>

    <sch:pattern>
        <sch:title>Rules for program hierarchy</sch:title>
        <sch:p>An producer may use this hierarchy to make clear hierarchical relationships between programs.</sch:p>
        <sch:rule context="st:HierarchicalCodelist[@id='HCL_CONFIGURATION_HIERARCHIES']/st:Hierarchy[@id='HY_PROGRAM']//st:CodeRef" 
            role="Program nodes">
            <sch:assert test="key('volatile_aliases',st:CodelistAliasRef)/st:CodelistID='CL_PROGRAM'">It is an error if a
                child node in this hierarchy does not have a CodelistAliasRef element which is an alias of 'CL_PROGRAM'.</sch:assert>
        </sch:rule>
    </sch:pattern>
    
    <sch:pattern>
        <sch:title>Rules for geographical place name hierarchy</sch:title>
        <sch:p>A DSD may specify a geographical place name dimension which takes its values from ISO/3166-2 codes in the code list 
            with id=CL_GEOGRAPHIC_PLACE_NAME. This hierarchy provides producers with a mechanism to structure the place name codes.</sch:p>
        <sch:rule context="st:HierarchicalCodelist[@id='HCL_CONFIGURATION_HIERARCHIES']/st:Hierarchy[@id='HY_GEOGRAPHIC_PLACE_NAME']//st:CodeRef" 
            role="Geographical place name nodes">
            <sch:assert
                test="key('volatile_aliases',st:CodelistAliasRef)/st:CodelistID='CL_GEOGRAPHIC_PLACE_NAME'"
                >HY_GEOGRAPHIC_PLACE_NAME is a tree of place name nodes. It is an error if a child
                node in this hierarchy does not have a CodelistAliasRef which is an alias of
                'CL_GEOGRAPHIC_PLACE_NAME'.</sch:assert>
        </sch:rule>
    </sch:pattern>
    
    <sch:pattern>
        <sch:title>Rules for the indicator hierarchy</sch:title>
        <sch:p>A DSD may specify how indicators are calculated by using the indicator hierarchy to
            specify other indicators to use as numerator and denominator.</sch:p>
        <sch:rule
            context="st:HierarchicalCodelist[@id='HCL_CONFIGURATION_HIERARCHIES']/st:Hierarchy[@id='HY_INDICATOR']/st:CodeRef"
            role="Indicator nodes">
            <sch:assert
                test="key('volatile_aliases',st:CodelistAliasRef)/st:CodelistID='CL_INDICATOR'"
                >Immediate child nodes of the hierarchy represent the indicators to be calculated.
                It is an error if an immediate child node is not an indicator node
                ('CodelistAliasRef points to CL_INDICATOR').</sch:assert>
            <sch:assert
                test="count(st:CodeRef[key('volatile_aliases',st:CodelistAliasRef)/st:CodelistID='CL_INDICATOR'])=2"
                >Each top-level indicator node shall have exactly two child indicator nodes
                representing the operand indicators of the calculation.</sch:assert>
        </sch:rule>
        <sch:rule
            context="st:HierarchicalCodelist[@id='HCL_CONFIGURATION_HIERARCHIES']/st:Hierarchy[@id='HY_INDICATOR']/st:CodeRef/st:CodeRef/st:CodeRef"
            role="Operand types">
            <sch:assert
                test="key('volatile_aliases',st:CodelistAliasRef)/st:CodelistID='CL_OPERAND'">Each
                operand indicator node shall have a child node indicating the type of the operand
                ('CodelistAliasRef points to CL_OPERAND').</sch:assert>
        </sch:rule>
    </sch:pattern>
    
    <sch:pattern>
        <sch:title>Rules for KeyFamily</sch:title>
        <sch:rule context="st:KeyFamily/st:Components" role="Required components">
            <sch:assert flag="fatal" test="count(st:Dimension[@conceptRef='INDICATOR'])=1">A
                KeyFamily shall have exactly one Dimension with a conceptRef attribute with a value
                of 'INDICATOR'</sch:assert>
            <sch:assert flag="fatal" test="count(st:PrimaryMeasure)=1">A KeyFamily shall have
                exactly one Dimension with a conceptRef attribute with a value of
                'OBS_VALUE'</sch:assert>
        </sch:rule>
        <sch:rule context="st:Dimension[@conceptRef='INDICATOR']" role="The INDICATOR dimension">
            <sch:assert flag="fatal"
                test="@codelist='CL_INDICATOR' and @conceptSchemeRef='CS_COMMON' and @conceptSchemeAgency='SDMX-HD'"
                > The INDICATOR Dimension shall have a codelist attribute of value 'CL_INDICATOR', a
                conceptSchemeRef attribute of value 'CS_COMMON' and a conceptSchemeAgency attribute
                of value 'SDMX-HD'</sch:assert>
        </sch:rule>
        <sch:rule context="st:PrimaryMeasure" role="The PrimaryMeasure">
            <sch:assert flag="fatal"
                test="@conceptRef='OBS_VALUE' and @conceptSchemeRef='CS_COMMON' and @conceptSchemeAgency='SDMX-HD'"
                > The PrimaryMeasure shall have a conceptRef attribute of value 'OBS_VALUE', a
                conceptSchemeRef attribute of value 'CS_COMMON' and a conceptSchemeAgency attribute
                of value 'SDMX-HD'</sch:assert>
        </sch:rule>
        <sch:rule context="st:KeyFamily/st:Components/st:Dimension"
            role="Disaggregation Dimension elements">
            <sch:assert flag="fatal" test="count(@codelist)=1"> It is an error for a Dimension not
                to have a codelist attribute </sch:assert>
        </sch:rule>
    </sch:pattern>

</sch:schema>