<?xml version="1.0" encoding="UTF-8"?>
<?xml-stylesheet type="text/xsl" href="sch2html.xsl"?>
<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xhtml="http://www.w3.org/1999/xhtml">

    <sch:ns uri="http://www.SDMX.org/resources/SDMXML/schemas/v2_0/message" prefix="msg"/>
    <sch:ns uri="http://www.SDMX.org/resources/SDMXML/schemas/v2_0/structure" prefix="st"/>
    
    <sch:title>Rules for an SDMX-HD Structure Message</sch:title>
    <sch:p>This document contains rules about an SDMX-HD Structure Message. It does not replicate
        existing constraints - which should be validated with the SDMX W3C schemas. It does provide
        additional constraints which conforming producers of an SDMX-HD DSD and an SDMX-HD MSD shall
        comply with.</sch:p>

    <sch:pattern id="ids">
        <sch:title>Rules relating to the naming of element ids</sch:title>
        <sch:rule context="st:CodeList" role="CodeLists">
            <sch:assert test="substring(@id,1,3)='CL_'"> A CodeList id attribute shall be prefixed
                by the string "CL_".</sch:assert>
        </sch:rule>
        <sch:rule context="st:ConceptScheme" role="ConceptSchemes">
            <sch:assert test="substring(@id,1,3)='CS_'"> A ConceptScheme id attribute shall be
                prefixed by the string "CS_".</sch:assert>
        </sch:rule>
        <sch:rule context="st:HierarchicalCodelist" role="HierarchicalCodelists">
            <sch:assert test="substring(@id,1,4)='HCL_'"> A HierarchicalCodelist id attribute shall
                be prefixed by the string "HCL_".</sch:assert>
        </sch:rule>
        <sch:rule context="st:Hierarchy" role="Hierarchies">
            <sch:assert test="substring(@id,1,3)='HY_'"> A Hierarchy id attribute shall be prefixed
                by the string "HY_".</sch:assert>
        </sch:rule>
    </sch:pattern>

<sch:pattern abstract="true" id="structure-element">
    <sch:title>Structure elements which are external references</sch:title>
    <sch:rule context="$element[@isExternalReference='true']"
        role="Structure element">
        <sch:let name="extElement" value="document(@uri)/msg:Structure/$parent/$element"/>
            <sch:assert
                test="$extElement/@id=@id 
            and $extElement/@agencyID=@agencyID 
            and $extElement/@version=@version 
            and $extElement/@isFinal=@isFinal"
                >Elements may be declared with external references within a DSD document. It is an
                error if the uri attribute does not resolve to a resource with an element identified
                by the id, agencyID, version, isFinal and urn attributes.</sch:assert>
    </sch:rule>
    </sch:pattern>
    
    <sch:pattern is-a="structure-element">
        <sch:title>CodeLists</sch:title>
        <sch:param name="element" value="st:CodeList"/>
        <sch:param name="parent" value="msg:CodeLists"/>
    </sch:pattern>

    <sch:pattern is-a="structure-element">
        <sch:title>ConceptSchemes</sch:title>
        <sch:param name="element" value="st:ConceptScheme"/>
        <sch:param name="parent" value="msg:Concepts"/>
    </sch:pattern>
    
    <sch:pattern is-a="structure-element">
        <sch:title>HierarchicalCodelists</sch:title>
        <sch:param name="element" value="st:HierarchicalCodelist"/>
        <sch:param name="parent" value="msg:HierarchicalCodelists"/>
    </sch:pattern>
    
    <sch:pattern is-a="structure-element">
        <sch:title>OrganisationSchemes</sch:title>
        <sch:param name="element" value="st:OrganisationScheme"/>
        <sch:param name="parent" value="msg:OrganisationSchemes"/>
    </sch:pattern>
    

    <sch:pattern abstract="true" id="sdmx-hd-structure-element">
        <sch:title>Specific rules relating to SDMX-HD Common structure elements</sch:title>
        <sch:p>A conforming producer shall not alter the contents of elements with agencyID
            attribute of 'SDMX-HD' in any way. Conforming producers of SDMX-HD zip packages should
            observe the following conventions regarding paths and names of external references.</sch:p>
        <sch:rule context="$element[@isExternalReference='true' and @agencyID='SDMX-HD']"
            role="External reference URLs">
            <sch:let name="rel-path" value="concat('COMMON/SDMX-HD/v1.0/$el-path/',concat(@id,'+',@agencyID,'+',@version,'.xml'))"/>
            <sch:assert
                test="(contains(@uri,':')=false) and (@uri=$rel-path or @uri=concat('./',$rel-path))"
                >External references to elements with agencyID='SDMX-HD' shall take the form of
                "COMMON/SDMX-HD/v1.0/" followed by either "codelists", "conceptschemes",
                "hierarchies" or "organisationschemes" followed by the filename. The filename part
                of the uri shall be a concatenation of the id, agencyID and version attributes
                separated by a '+' character, and should have a suffix of ".xml". </sch:assert>
        </sch:rule>
    </sch:pattern>
    
    <sch:pattern is-a="sdmx-hd-structure-element">
        <sch:title>CodeLists</sch:title>
        <sch:param name="element" value="st:CodeList"/>
        <sch:param name="el-path" value="codelists"/>
    </sch:pattern>
    
    <sch:pattern is-a="sdmx-hd-structure-element">
        <sch:title>ConceptSchemes</sch:title>
        <sch:param name="element" value="st:ConceptScheme"/>
        <sch:param name="el-path" value="conceptschemes"/>
    </sch:pattern>
    
    <sch:pattern is-a="sdmx-hd-structure-element">
        <sch:title>HierarchicalCodelists</sch:title>
        <sch:param name="element" value="st:HierarchicalCodelist"/>
        <sch:param name="el-path" value="hierarchies"/>
    </sch:pattern>
    
    <sch:pattern is-a="sdmx-hd-structure-element">
        <sch:title>OrganisationSchemes</sch:title>
        <sch:param name="element" value="st:OrganisationScheme"/>
        <sch:param name="el-path" value="organisationschemes"/>
    </sch:pattern>
    
</sch:schema>